extends Node2D
class_name PenguGroup

@export var n_penguins = 1
@export var ice: IceFloe

@onready var timer = $Timer

var penguin_scene: PackedScene = load("res://NPC/penguin/penguin.tscn")
var penguins : Array[Penguin] = []

func _ready() -> void:
	Globals.pengu_group = self
	timer.connect("timeout", Callable(self, "_on_timer_timeout"))
	start_random_timer()
	
	for i in range(n_penguins):
		var p = penguin_scene.instantiate()
		penguins.append(p)
		add_child(p)
		p.global_position = get_random_point_in_Polygon(ice.get_child(0)) + ice.global_position
		p.make_child()

static func get_random_point_in_Polygon(polygon: Polygon2D) -> Vector2:
	var points = polygon.polygon
	var triangles = Geometry2D.triangulate_polygon(points)

	var tri_index = (randi() % (triangles.size() / 3)) * 3
	var p1 = points[triangles[tri_index]]
	var p2 = points[triangles[tri_index + 1]]
	var p3 = points[triangles[tri_index + 2]]

	return get_random_point_in_triangle(p1, p2, p3)
	

static func get_random_point_in_triangle(a: Vector2, b: Vector2, c: Vector2) -> Vector2:
	var r1 = randf()
	var r2 = sqrt(randf())  # Square root for uniform distribution
	var p = (1 - r2) * a + (r2 * (1 - r1)) * b + (r2 * r1) * c
	return p

func notifiy_pengu_scholle_changed():
	for p in penguins:
		p.make_child()
		
func notify_pengu_suspend():
	for p in penguins:
		p.suspend()
		
func _on_timer_timeout():
	var coin = randi_range(0, 2)
	var pengu_to_move: Penguin
	if (coin == 0):
		pengu_to_move = penguins[randf_range(0, len(penguins)-1)]
	else:
		var p = []
		for pengu in penguins:
			if(is_instance_valid(pengu)):
				p.append(pengu.cluster_score)
			else:
				break
		var max = p.max()
		var index = p.find(max)
		
		if(is_instance_valid(penguins[index])):
			pengu_to_move = penguins[index]	
	if is_instance_valid(pengu_to_move) && pengu_to_move.get_parent() is IceFloe:
		pengu_to_move.set_target(get_random_point_in_Polygon(pengu_to_move.get_parent().get_child(0)));
	
	start_random_timer()

func start_random_timer():
	var random_time = randf_range(0.5, 3.0)
	timer.start(random_time)
